@@ -8,7 +8,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2000-2005 Julian Seward 
+   Copyright (C) 2000-2006 Julian Seward 
       jseward@acm.org
 
    This program is free software; you can redistribute it and/or
@@ -44,18 +44,17 @@
    traces, so we want the name to be (hopefully!) meaningful to users.
    ------------------------------------------------------------------ */
 
-#include "valgrind.h"               // for VALGRIND_NON_SIMD_CALL[12]
-#include "coregrind.h"              // for VALGRIND_INTERNAL_PRINTF
-
 #include "pub_core_basics.h"
+#include "pub_core_clreq.h"         // for VALGRIND_INTERNAL_PRINTF,
+                                    //   VALGRIND_NON_SIMD_CALL[12]
 #include "pub_core_debuginfo.h"     // needed for pub_core_redir.h :(
 #include "pub_core_mallocfree.h"    // for VG_MIN_MALLOC_SZB, VG_AR_CLIENT
-#include "pub_core_redir.h"         // for VG_REPLACE_FUNCTION
+#include "pub_core_redir.h"         // for VG_REDIRECT_FUNCTION_*
 #include "pub_core_replacemalloc.h"
 
 /* Some handy Z-encoded names */
 #define  m_libstc_plus_plus_star  libstdcZpZpZa   // libstdc++*
-#define  m_libc_dot_so_dot_6      libcZdsoZd6     // libc.so.6
+#define  m_libc_dot_so_star       libcZdsoZa      // libc.so*
 //#define  m_libpgc_dot_so          libpgcZdso      // libpgc.so
 
 /* 2 Apr 05: the Portland Group compiler, which uses cfront/ARM style
@@ -91,10 +90,11 @@ static void init(void) __attribute__((constructor));
    __builtin_delete, calloc, realloc, memalign, and friends.
 
    None of these functions are called directly - they are not meant to
-   be found by the dynamic linker.  But ALL client calls to malloc() and
-   friends wind up here eventually.  They get called because vg_replace_malloc
-   installs a bunch of code redirects which causes Valgrind to use these
-   functions rather than the ones they're replacing.
+   be found by the dynamic linker.  But ALL client calls to malloc()
+   and friends wind up here eventually.  They get called because
+   vg_replace_malloc installs a bunch of code redirects which causes
+   Valgrind to use these functions rather than the ones they're
+   replacing.
 */
 
 /* Generate a replacement for 'fnname' in object 'soname', which calls
@@ -102,13 +102,13 @@ static void init(void) __attribute__((constructor));
 */
 #define ALLOC_or_NULL(soname, fnname, vg_replacement) \
    \
-   void* VG_REPLACE_FUNCTION(soname,fnname) (SizeT n); \
-   void* VG_REPLACE_FUNCTION(soname,fnname) (SizeT n)  \
+   void* VG_REPLACE_FUNCTION_ZU(soname,fnname) (SizeT n); \
+   void* VG_REPLACE_FUNCTION_ZU(soname,fnname) (SizeT n)  \
    { \
       void* v; \
       \
-      MALLOC_TRACE(#fnname "(%llu)", (ULong)n ); \
       if (!init_done) init(); \
+      MALLOC_TRACE(#fnname "(%llu)", (ULong)n ); \
       \
       v = (void*)VALGRIND_NON_SIMD_CALL1( info.tl_##vg_replacement, n ); \
       MALLOC_TRACE(" = %p", v ); \
@@ -122,13 +122,13 @@ static void init(void) __attribute__((constructor));
 */
 #define ALLOC_or_BOMB(soname, fnname, vg_replacement)  \
    \
-   void* VG_REPLACE_FUNCTION(soname,fnname) (SizeT n); \
-   void* VG_REPLACE_FUNCTION(soname,fnname) (SizeT n)  \
+   void* VG_REPLACE_FUNCTION_ZU(soname,fnname) (SizeT n); \
+   void* VG_REPLACE_FUNCTION_ZU(soname,fnname) (SizeT n)  \
    { \
       void* v; \
       \
-      MALLOC_TRACE(#fnname "(%llu)", (ULong)n ); \
       if (!init_done) init(); \
+      MALLOC_TRACE(#fnname "(%llu)", (ULong)n );        \
       \
       v = (void*)VALGRIND_NON_SIMD_CALL1( info.tl_##vg_replacement, n ); \
       MALLOC_TRACE(" = %p", v ); \
@@ -146,27 +146,26 @@ static void init(void) __attribute__((constructor));
 
 // malloc
 ALLOC_or_NULL(m_libstc_plus_plus_star, malloc,      malloc);
-ALLOC_or_NULL(m_libc_dot_so_dot_6,     malloc,      malloc);
+ALLOC_or_NULL(m_libc_dot_so_star,      malloc,      malloc);
 //ALLOC_or_NULL(m_libpgc_dot_so,         malloc,      malloc);
 
 
-// operator new(unsigned int), unmangled for some bizarre reason
+// operator new(unsigned int), not mangled (for gcc 2.96)
 ALLOC_or_BOMB(m_libstc_plus_plus_star, builtin_new,    __builtin_new);
-ALLOC_or_BOMB(m_libc_dot_so_dot_6,     builtin_new,    __builtin_new);
+ALLOC_or_BOMB(m_libc_dot_so_star,      builtin_new,    __builtin_new);
 
 ALLOC_or_BOMB(m_libstc_plus_plus_star, __builtin_new,  __builtin_new);
-ALLOC_or_BOMB(m_libc_dot_so_dot_6,     __builtin_new,  __builtin_new);
-
+ALLOC_or_BOMB(m_libc_dot_so_star,      __builtin_new,  __builtin_new);
 
 // operator new(unsigned int), GNU mangling, 32-bit platforms
 // operator new(unsigned long), GNU mangling, 64-bit platforms
 #if VG_WORDSIZE == 4
  ALLOC_or_BOMB(m_libstc_plus_plus_star, _Znwj,          __builtin_new);
- ALLOC_or_BOMB(m_libc_dot_so_dot_6,     _Znwj,          __builtin_new);
+ ALLOC_or_BOMB(m_libc_dot_so_star,      _Znwj,          __builtin_new);
 #endif
 #if VG_WORDSIZE == 8
  ALLOC_or_BOMB(m_libstc_plus_plus_star, _Znwm,          __builtin_new);
- ALLOC_or_BOMB(m_libc_dot_so_dot_6,     _Znwm,          __builtin_new);
+ ALLOC_or_BOMB(m_libc_dot_so_star,      _Znwm,          __builtin_new);
 #endif
 
 
@@ -178,28 +177,27 @@ ALLOC_or_BOMB(m_libc_dot_so_dot_6,     __builtin_new,  __builtin_new);
 // operator new(unsigned long, std::nothrow_t const&), GNU mangling, 64-bit
 #if VG_WORDSIZE == 4
  ALLOC_or_NULL(m_libstc_plus_plus_star, _ZnwjRKSt9nothrow_t,  __builtin_new);
- ALLOC_or_NULL(m_libc_dot_so_dot_6,     _ZnwjRKSt9nothrow_t,  __builtin_new);
+ ALLOC_or_NULL(m_libc_dot_so_star,      _ZnwjRKSt9nothrow_t,  __builtin_new);
 #endif
 #if VG_WORDSIZE == 8
  ALLOC_or_NULL(m_libstc_plus_plus_star, _ZnwmRKSt9nothrow_t,  __builtin_new);
- ALLOC_or_NULL(m_libc_dot_so_dot_6,     _ZnwmRKSt9nothrow_t,  __builtin_new);
+ ALLOC_or_NULL(m_libc_dot_so_star,      _ZnwmRKSt9nothrow_t,  __builtin_new);
 #endif
 
 
-// operator new[](unsigned int), unmangled for some bizarre reason
+// operator new[](unsigned int), not mangled (for gcc 2.96)
 ALLOC_or_BOMB(m_libstc_plus_plus_star, __builtin_vec_new, __builtin_vec_new );
-ALLOC_or_BOMB(m_libc_dot_so_dot_6,     __builtin_vec_new, __builtin_vec_new );
-
+ALLOC_or_BOMB(m_libc_dot_so_star,      __builtin_vec_new, __builtin_vec_new );
 
 // operator new[](unsigned int), GNU mangling, 32-bit platforms
 // operator new[](unsigned long), GNU mangling, 64-bit platforms
 #if VG_WORDSIZE == 4
  ALLOC_or_BOMB(m_libstc_plus_plus_star, _Znaj,             __builtin_vec_new );
- ALLOC_or_BOMB(m_libc_dot_so_dot_6,     _Znaj,             __builtin_vec_new );
+ ALLOC_or_BOMB(m_libc_dot_so_star,      _Znaj,             __builtin_vec_new );
 #endif
 #if VG_WORDSIZE == 8
  ALLOC_or_BOMB(m_libstc_plus_plus_star, _Znam,             __builtin_vec_new );
- ALLOC_or_BOMB(m_libc_dot_so_dot_6,     _Znam,             __builtin_vec_new );
+ ALLOC_or_BOMB(m_libc_dot_so_star,     _Znam,             __builtin_vec_new );
 #endif
 
 
@@ -207,11 +205,11 @@ ALLOC_or_BOMB(m_libc_dot_so_dot_6,     __builtin_vec_new, __builtin_vec_new );
 // operator new[](unsigned long, std::nothrow_t const&), GNU mangling, 64-bit
 #if VG_WORDSIZE == 4
  ALLOC_or_NULL(m_libstc_plus_plus_star, _ZnajRKSt9nothrow_t, __builtin_vec_new );
- ALLOC_or_NULL(m_libc_dot_so_dot_6,     _ZnajRKSt9nothrow_t, __builtin_vec_new );
+ ALLOC_or_NULL(m_libc_dot_so_star,      _ZnajRKSt9nothrow_t, __builtin_vec_new );
 #endif
 #if VG_WORDSIZE == 8
  ALLOC_or_NULL(m_libstc_plus_plus_star, _ZnamRKSt9nothrow_t, __builtin_vec_new );
- ALLOC_or_NULL(m_libc_dot_so_dot_6,     _ZnamRKSt9nothrow_t, __builtin_vec_new );
+ ALLOC_or_NULL(m_libc_dot_so_star,      _ZnamRKSt9nothrow_t, __builtin_vec_new );
 #endif
 
 
@@ -220,99 +218,102 @@ ALLOC_or_BOMB(m_libc_dot_so_dot_6,     __builtin_vec_new, __builtin_vec_new );
 */
 #define FREE(soname, fnname, vg_replacement) \
    \
-   void VG_REPLACE_FUNCTION(soname,fnname) (void *p); \
-   void VG_REPLACE_FUNCTION(soname,fnname) (void *p)  \
+   void VG_REPLACE_FUNCTION_ZU(soname,fnname) (void *p); \
+   void VG_REPLACE_FUNCTION_ZU(soname,fnname) (void *p)  \
    { \
+      if (!init_done) init(); \
       MALLOC_TRACE(#vg_replacement "(%p)", p ); \
       if (p == NULL)  \
          return; \
-      if (!init_done) init(); \
       (void)VALGRIND_NON_SIMD_CALL1( info.tl_##vg_replacement, p ); \
    }
 
 // free
 FREE(m_libstc_plus_plus_star,  free,                 free );
-FREE(m_libc_dot_so_dot_6,      free,                 free );
+FREE(m_libc_dot_so_star,       free,                 free );
 
 // cfree
 FREE(m_libstc_plus_plus_star,  cfree,                free );
-FREE(m_libc_dot_so_dot_6,      cfree,                free );
+FREE(m_libc_dot_so_star,       cfree,                free );
 
-// do we really need these?
+// operator delete(void*), not mangled (for gcc 2.96)
 FREE(m_libstc_plus_plus_star,  __builtin_delete,     __builtin_delete );
-FREE(m_libc_dot_so_dot_6,      __builtin_delete,     __builtin_delete );
+FREE(m_libc_dot_so_star,       __builtin_delete,     __builtin_delete );
 
 // operator delete(void*), GNU mangling
 FREE(m_libstc_plus_plus_star,  _ZdlPv,               __builtin_delete );
-FREE(m_libc_dot_so_dot_6,      _ZdlPv,               __builtin_delete );
+FREE(m_libc_dot_so_star,       _ZdlPv,               __builtin_delete );
 
 // operator delete(void*, std::nothrow_t const&), GNU mangling
 FREE(m_libstc_plus_plus_star, _ZdlPvRKSt9nothrow_t,  __builtin_delete );
-FREE(m_libc_dot_so_dot_6,     _ZdlPvRKSt9nothrow_t,  __builtin_delete );
+FREE(m_libc_dot_so_star,      _ZdlPvRKSt9nothrow_t,  __builtin_delete );
+
+// operator delete[](void*), not mangled (for gcc 2.96)
+FREE(m_libstc_plus_plus_star,  __builtin_vec_delete, __builtin_vec_delete );
+FREE(m_libc_dot_so_star,       __builtin_vec_delete, __builtin_vec_delete );
 
 // operator delete[](void*), GNU mangling
-FREE(m_libstc_plus_plus_star,  __builtin_vec_delete, __builtin_vec_delete );
-FREE(m_libc_dot_so_dot_6,      __builtin_vec_delete, __builtin_vec_delete );
 FREE(m_libstc_plus_plus_star,  _ZdaPv,               __builtin_vec_delete );
-FREE(m_libc_dot_so_dot_6,      _ZdaPv,               __builtin_vec_delete );
+FREE(m_libc_dot_so_star,       _ZdaPv,               __builtin_vec_delete );
 
 // operator delete[](void*, std::nothrow_t const&), GNU mangling
 FREE(m_libstc_plus_plus_star,  _ZdaPvRKSt9nothrow_t, __builtin_vec_delete );
-FREE(m_libc_dot_so_dot_6,      _ZdaPvRKSt9nothrow_t, __builtin_vec_delete );
+FREE(m_libc_dot_so_star,       _ZdaPvRKSt9nothrow_t, __builtin_vec_delete );
 
 
 #define CALLOC(soname, fnname) \
    \
-   void* VG_REPLACE_FUNCTION(soname,fnname) ( SizeT nmemb, SizeT size ); \
-   void* VG_REPLACE_FUNCTION(soname,fnname) ( SizeT nmemb, SizeT size )  \
+   void* VG_REPLACE_FUNCTION_ZU(soname,fnname) ( SizeT nmemb, SizeT size ); \
+   void* VG_REPLACE_FUNCTION_ZU(soname,fnname) ( SizeT nmemb, SizeT size )  \
    { \
       void* v; \
       \
+      if (!init_done) init(); \
       MALLOC_TRACE("calloc(%llu,%llu)", (ULong)nmemb, (ULong)size ); \
       \
-      if (!init_done) init(); \
       v = (void*)VALGRIND_NON_SIMD_CALL2( info.tl_calloc, nmemb, size ); \
       MALLOC_TRACE(" = %p", v ); \
       return v; \
    }
 
-CALLOC(m_libc_dot_so_dot_6, calloc);
+CALLOC(m_libc_dot_so_star, calloc);
 
 
 #define REALLOC(soname, fnname) \
    \
-   void* VG_REPLACE_FUNCTION(soname,fnname) ( void* ptrV, SizeT new_size );\
-   void* VG_REPLACE_FUNCTION(soname,fnname) ( void* ptrV, SizeT new_size ) \
+   void* VG_REPLACE_FUNCTION_ZU(soname,fnname) ( void* ptrV, SizeT new_size );\
+   void* VG_REPLACE_FUNCTION_ZU(soname,fnname) ( void* ptrV, SizeT new_size ) \
    { \
       void* v; \
       \
+      if (!init_done) init(); \
       MALLOC_TRACE("realloc(%p,%llu)", ptrV, (ULong)new_size ); \
       \
       if (ptrV == NULL) \
          /* We need to call a malloc-like function; so let's use \
             one which we know exists. */ \
-         return VG_REPLACE_FUNCTION(libcZdsoZd6,malloc) (new_size); \
+         return VG_REPLACE_FUNCTION_ZU(libcZdsoZa,malloc) (new_size); \
       if (new_size <= 0) { \
-         VG_REPLACE_FUNCTION(libcZdsoZd6,free)(ptrV); \
+         VG_REPLACE_FUNCTION_ZU(libcZdsoZa,free)(ptrV); \
          MALLOC_TRACE(" = 0"); \
          return NULL; \
       } \
-      if (!init_done) init(); \
       v = (void*)VALGRIND_NON_SIMD_CALL2( info.tl_realloc, ptrV, new_size ); \
       MALLOC_TRACE(" = %p", v ); \
       return v; \
    }
 
-REALLOC(m_libc_dot_so_dot_6, realloc);
+REALLOC(m_libc_dot_so_star, realloc);
 
 
 #define MEMALIGN(soname, fnname) \
    \
-   void* VG_REPLACE_FUNCTION(soname,fnname) ( SizeT alignment, SizeT n ); \
-   void* VG_REPLACE_FUNCTION(soname,fnname) ( SizeT alignment, SizeT n )  \
+   void* VG_REPLACE_FUNCTION_ZU(soname,fnname) ( SizeT alignment, SizeT n ); \
+   void* VG_REPLACE_FUNCTION_ZU(soname,fnname) ( SizeT alignment, SizeT n )  \
    { \
       void* v; \
       \
+      if (!init_done) init(); \
       MALLOC_TRACE("memalign(al %llu, size %llu)", \
                    (ULong)alignment, (ULong)n ); \
       \
@@ -323,45 +324,81 @@ REALLOC(m_libc_dot_so_dot_6, realloc);
       /* Round up to nearest power-of-two if necessary (like glibc). */ \
       while (0 != (alignment & (alignment - 1))) alignment++; \
       \
-      if (!init_done) init(); \
       v = (void*)VALGRIND_NON_SIMD_CALL2( info.tl_memalign, alignment, n ); \
       MALLOC_TRACE(" = %p", v ); \
       return v; \
    }
 
-MEMALIGN(m_libc_dot_so_dot_6, memalign);
+MEMALIGN(m_libc_dot_so_star, memalign);
 
 
 #define VALLOC(soname, fnname) \
    \
-   void* VG_REPLACE_FUNCTION(soname,fnname) ( SizeT size ); \
-   void* VG_REPLACE_FUNCTION(soname,fnname) ( SizeT size )  \
+   void* VG_REPLACE_FUNCTION_ZU(soname,fnname) ( SizeT size ); \
+   void* VG_REPLACE_FUNCTION_ZU(soname,fnname) ( SizeT size )  \
    { \
-      return VG_REPLACE_FUNCTION(libcZdsoZd6,memalign)(VKI_PAGE_SIZE, size); \
+      return VG_REPLACE_FUNCTION_ZU(libcZdsoZa,memalign)(VKI_PAGE_SIZE, size); \
    }
 
-VALLOC(m_libc_dot_so_dot_6, valloc);
+VALLOC(m_libc_dot_so_star, valloc);
 
 
 /* Various compatibility wrapper functions, for glibc and libstdc++. */
 
 #define MALLOPT(soname, fnname) \
    \
-   int VG_REPLACE_FUNCTION(soname, fnname) ( int cmd, int value ); \
-   int VG_REPLACE_FUNCTION(soname, fnname) ( int cmd, int value )  \
+   int VG_REPLACE_FUNCTION_ZU(soname, fnname) ( int cmd, int value ); \
+   int VG_REPLACE_FUNCTION_ZU(soname, fnname) ( int cmd, int value )  \
    { \
       /* In glibc-2.2.4, 1 denotes a successful return value for \
          mallopt */ \
       return 1; \
    }
 
-MALLOPT(m_libc_dot_so_dot_6, mallopt);
+MALLOPT(m_libc_dot_so_star, mallopt);
+
+
+// Documentation says:
+//   malloc_trim(size_t pad);
+// 
+//   If possible, gives memory back to the system (via negative arguments to
+//   sbrk) if there is unused memory at the `high' end of the malloc pool.
+//   You can call this after freeing large blocks of memory to potentially
+//   reduce the system-level memory requirements of a program. However, it
+//   cannot guarantee to reduce memory.  Under some allocation patterns,
+//   some large free blocks of memory will be locked between two used
+//   chunks, so they cannot be given back to the system.
+// 
+//   The `pad' argument to malloc_trim represents the amount of free
+//   trailing space to leave untrimmed. If this argument is zero, only the
+//   minimum amount of memory to maintain internal data structures will be
+//   left (one page or less). Non-zero arguments can be supplied to maintain
+//   enough trailing space to service future expected allocations without
+//   having to re-obtain memory from the system.
+// 
+//   Malloc_trim returns 1 if it actually released any memory, else 0. On
+//   systems that do not support "negative sbrks", it will always return 0. 
+//
+// For simplicity, we always return 0.
+#define MALLOC_TRIM(soname, fnname) \
+   \
+   int VG_REPLACE_FUNCTION_ZU(soname, fnname) ( SizeT pad ); \
+   int VG_REPLACE_FUNCTION_ZU(soname, fnname) ( SizeT pad )  \
+   { \
+      /* 0 denotes that malloc_trim() either wasn't able \
+         to do anything, or was not implemented */ \
+      return 0; \
+   }
+
+MALLOC_TRIM(m_libc_dot_so_star, malloc_trim);
 
 
 #define POSIX_MEMALIGN(soname, fnname) \
    \
-   int VG_REPLACE_FUNCTION(soname, fnname) ( void **memptr, SizeT alignment, SizeT size ); \
-   int VG_REPLACE_FUNCTION(soname, fnname) ( void **memptr, SizeT alignment, SizeT size )  \
+   int VG_REPLACE_FUNCTION_ZU(soname, fnname) ( void **memptr, \
+                                                 SizeT alignment, SizeT size ); \
+   int VG_REPLACE_FUNCTION_ZU(soname, fnname) ( void **memptr, \
+                                                 SizeT alignment, SizeT size )  \
    { \
       void *mem; \
       \
@@ -371,7 +408,7 @@ MALLOPT(m_libc_dot_so_dot_6, mallopt);
           || (alignment & (alignment - 1)) != 0) \
          return VKI_EINVAL; \
       \
-      mem = VG_REPLACE_FUNCTION(libcZdsoZd6,memalign)(alignment, size); \
+      mem = VG_REPLACE_FUNCTION_ZU(libcZdsoZa,memalign)(alignment, size); \
       \
       if (mem != NULL) { \
         *memptr = mem; \
@@ -381,21 +418,21 @@ MALLOPT(m_libc_dot_so_dot_6, mallopt);
       return VKI_ENOMEM; \
    }
 
-POSIX_MEMALIGN(m_libc_dot_so_dot_6, posix_memalign);
+POSIX_MEMALIGN(m_libc_dot_so_star, posix_memalign);
 
 
 #define MALLOC_USABLE_SIZE(soname, fnname) \
    \
-   int VG_REPLACE_FUNCTION(soname, fnname) ( void* p ); \
-   int VG_REPLACE_FUNCTION(soname, fnname) ( void* p )  \
+   int VG_REPLACE_FUNCTION_ZU(soname, fnname) ( void* p ); \
+   int VG_REPLACE_FUNCTION_ZU(soname, fnname) ( void* p )  \
    {  \
       SizeT pszB; \
       \
+      if (!init_done) init(); \
       MALLOC_TRACE("malloc_usable_size(%p)", p ); \
       if (NULL == p) \
          return 0; \
       \
-      if (!init_done) init(); \
       pszB = (SizeT)VALGRIND_NON_SIMD_CALL2( info.arena_payload_szB, \
                                              VG_AR_CLIENT, p ); \
       MALLOC_TRACE(" = %llu", (ULong)pszB ); \
@@ -403,7 +440,7 @@ POSIX_MEMALIGN(m_libc_dot_so_dot_6, posix_memalign);
       return pszB; \
    }
 
-MALLOC_USABLE_SIZE(m_libc_dot_so_dot_6, malloc_usable_size);
+MALLOC_USABLE_SIZE(m_libc_dot_so_star, malloc_usable_size);
 
 
 /* Bomb out if we get any of these. */
@@ -417,52 +454,33 @@ static void panic(const char *str)
 
 #define PANIC(soname, fnname) \
    \
-   void VG_REPLACE_FUNCTION(soname, fnname) ( void ); \
-   void VG_REPLACE_FUNCTION(soname, fnname) ( void )  \
+   void VG_REPLACE_FUNCTION_ZU(soname, fnname) ( void ); \
+   void VG_REPLACE_FUNCTION_ZU(soname, fnname) ( void )  \
    { \
       panic(#fnname); \
    }
 
-PANIC(m_libc_dot_so_dot_6, pvalloc);
-PANIC(m_libc_dot_so_dot_6, malloc_stats);
-PANIC(m_libc_dot_so_dot_6, malloc_trim);
-PANIC(m_libc_dot_so_dot_6, malloc_get_state);
-PANIC(m_libc_dot_so_dot_6, malloc_set_state);
-
-
-/* Yet another ugly hack.  Cannot include <malloc.h> because we
-   implement functions implemented there with different signatures.
-   This struct definition MUST match the system one. */
-
-/* SVID2/XPG mallinfo structure */
-struct mallinfo {
-   int arena;    /* total space allocated from system */
-   int ordblks;  /* number of non-inuse chunks */
-   int smblks;   /* unused -- always zero */
-   int hblks;    /* number of mmapped regions */
-   int hblkhd;   /* total space in mmapped regions */
-   int usmblks;  /* unused -- always zero */
-   int fsmblks;  /* unused -- always zero */
-   int uordblks; /* total allocated space */
-   int fordblks; /* total non-inuse space */
-   int keepcost; /* top-most, releasable (via malloc_trim) space */
-};
+PANIC(m_libc_dot_so_star, pvalloc);
+PANIC(m_libc_dot_so_star, malloc_stats);
+PANIC(m_libc_dot_so_star, malloc_get_state);
+PANIC(m_libc_dot_so_star, malloc_set_state);
 
+// mi must be static;  if it is auto then Memcheck thinks it is
+// uninitialised when used by the caller of this function, because Memcheck
+// doesn't know that the call to mallinfo fills in mi.
 #define MALLINFO(soname, fnname) \
    \
-   struct mallinfo VG_REPLACE_FUNCTION(soname, fnname) ( void ); \
-   struct mallinfo VG_REPLACE_FUNCTION(soname, fnname) ( void )  \
+   struct vg_mallinfo VG_REPLACE_FUNCTION_ZU(soname, fnname) ( void ); \
+   struct vg_mallinfo VG_REPLACE_FUNCTION_ZU(soname, fnname) ( void )  \
    { \
-      /* Should really try to return something a bit more meaningful */ \
-      UInt            i; \
-      struct mallinfo mi; \
-      UChar*          pmi = (UChar*)(&mi); \
-      for (i = 0; i < sizeof(mi); i++) \
-         pmi[i] = 0; \
+      static struct vg_mallinfo mi; \
+      if (!init_done) init(); \
+      MALLOC_TRACE("mallinfo()"); \
+      (void)VALGRIND_NON_SIMD_CALL1( info.mallinfo, &mi ); \
       return mi; \
    }
 
-MALLINFO(m_libc_dot_so_dot_6, mallinfo);
+MALLINFO(m_libc_dot_so_star, mallinfo);
 
 
 /* All the code in here is unused until this function is called */
@@ -476,8 +494,8 @@ static void init(void)
 
    init_done = 1;
 
-   VALGRIND_MAGIC_SEQUENCE(res, -1, VG_USERREQ__GET_MALLOCFUNCS, &info,
-                           0, 0, 0);
+   VALGRIND_DO_CLIENT_REQUEST(res, -1, VG_USERREQ__GET_MALLOCFUNCS, &info,
+                              0, 0, 0, 0);
 }
 
 /*--------------------------------------------------------------------*/
