@@ -1,7 +1,6 @@
 
 /*--------------------------------------------------------------------*/
-/*--- PPC32/Linux-specific kernel interface.                       ---*/
-/*---                                ppc32-linux/vki-ppc32-linux.h ---*/
+/*--- PPC32/Linux-specific kernel interface.     vki-ppc32-linux.h ---*/
 /*--------------------------------------------------------------------*/
 
 /*
@@ -152,16 +151,16 @@ typedef struct {
 #define VKI_SS_ONSTACK		1
 #define VKI_SS_DISABLE		2
 
-//.. struct vki_old_sigaction {
-//..         // [[Nb: a 'k' prefix is added to "sa_handler" because
-//..         // bits/sigaction.h (which gets dragged in somehow via signal.h)
-//..         // #defines it as something else.  Since that is done for glibc's
-//..         // purposes, which we don't care about here, we use our own name.]]
-//..         __vki_sighandler_t ksa_handler;
-//..         vki_old_sigset_t sa_mask;
-//..         unsigned long sa_flags;
-//..         __vki_sigrestore_t sa_restorer;
-//.. };
+struct vki_old_sigaction {
+        // [[Nb: a 'k' prefix is added to "sa_handler" because
+        // bits/sigaction.h (which gets dragged in somehow via signal.h)
+        // #defines it as something else.  Since that is done for glibc's
+        // purposes, which we don't care about here, we use our own name.]]
+        __vki_sighandler_t ksa_handler;
+        vki_old_sigset_t sa_mask;
+        unsigned long sa_flags;
+        __vki_sigrestore_t sa_restorer;
+};
 
 struct vki_sigaction {
         // [[See comment about extra 'k' above]]
@@ -199,11 +198,49 @@ struct vki_pt_regs {
         unsigned long dar;              /* Fault registers */
         unsigned long dsisr;            /* on 4xx/Book-E used for ESR */
         unsigned long result;           /* Result of a system call */
+
+        /* Not in kernel's definition, but apparently needed to stop
+           assertion at coredump-elf.c:267 firing.  These padding
+           words make the struct have the same size as a
+           'vki_elf_greg_t'.  See message from Ghassan Hammouri on
+           valgrind-developers on 6 April 06. */
+        unsigned long pad[4];
 };
 
 #define vki_user_regs_struct vki_pt_regs
 
-#define VKI_PT_R0		 0
+#define VKI_PT_R0		0
+#define VKI_PT_R1		1
+#define VKI_PT_R2		2
+#define VKI_PT_R3		3
+#define VKI_PT_R4		4
+#define VKI_PT_R5		5
+#define VKI_PT_R6		6
+#define VKI_PT_R7		7
+#define VKI_PT_R8		8
+#define VKI_PT_R9		9
+#define VKI_PT_R10		10
+#define VKI_PT_R11		11
+#define VKI_PT_R12		12
+#define VKI_PT_R13		13
+#define VKI_PT_R14		14
+#define VKI_PT_R15		15
+#define VKI_PT_R16		16
+#define VKI_PT_R17		17
+#define VKI_PT_R18		18
+#define VKI_PT_R19		19
+#define VKI_PT_R20		20
+#define VKI_PT_R21		21
+#define VKI_PT_R22		22
+#define VKI_PT_R23		23
+#define VKI_PT_R24		24
+#define VKI_PT_R25		25
+#define VKI_PT_R26		26
+#define VKI_PT_R27		27
+#define VKI_PT_R28		28
+#define VKI_PT_R29		29
+#define VKI_PT_R30		30
+#define VKI_PT_R31		31
 #define VKI_PT_NIP		32
 #define VKI_PT_MSR		33
 #define VKI_PT_ORIG_R3		34
@@ -237,6 +274,8 @@ struct vki_sigcontext {
 #define VKI_PROT_READ		0x1      /* page can be read */
 #define VKI_PROT_WRITE		0x2      /* page can be written */
 #define VKI_PROT_EXEC		0x4      /* page can be executed */
+#define VKI_PROT_GROWSDOWN	0x01000000	/* mprotect flag: extend change to start of growsdown vma */
+#define VKI_PROT_GROWSUP	0x02000000	/* mprotect flag: extend change to end of growsup vma */
 
 #define VKI_MAP_SHARED		0x01     /* Share changes */
 #define VKI_MAP_PRIVATE		0x02     /* Changes are private */
@@ -257,6 +296,7 @@ struct vki_sigcontext {
 #define VKI_O_TRUNC		01000		/* not fcntl */
 #define VKI_O_APPEND		02000
 #define VKI_O_NONBLOCK		04000
+#define VKI_O_LARGEFILE     0200000
 
 #define VKI_F_DUPFD		 0			/* dup */
 #define VKI_F_GETFD		 1			/* get close_on_exec */
@@ -302,7 +342,7 @@ struct vki_sigcontext {
 #define VKI_SIOCGSTAMP	0x8906          /* Get stamp */
 
 //----------------------------------------------------------------------
-// From linux-2.6.9/include/asm-ppc/stat.h
+// From linux-2.6.10/include/asm-ppc/stat.h
 //----------------------------------------------------------------------
 
 //.. #define VKI_S_IFMT		00170000
@@ -347,28 +387,29 @@ struct vki_stat {
 };
 
 struct vki_stat64 {
-   unsigned long	st_dev;
-   unsigned long	st_ino;
-   unsigned long	st_rdev;
-   long			st_size;
-   unsigned long	st_blocks;
+   unsigned long long   st_dev;
+   unsigned long long   st_ino;
+   unsigned int         st_mode;
+   unsigned int         st_nlink;
+   unsigned int         st_uid;
+   unsigned int         st_gid;
+   unsigned long long   st_rdev;
+   unsigned short int   __pad2;
+   long long            st_size;
+   long                 st_blksize;
 
-   unsigned int		st_mode;
-   unsigned int		st_uid;
-   unsigned int		st_gid;
-   unsigned int		st_blksize;
-   unsigned int		st_nlink;
-   unsigned int		__pad0;
-
-   unsigned long	st_atime;
-   unsigned long	st_atime_nsec;
-   unsigned long	st_mtime;
-   unsigned long	st_mtime_nsec;
-   unsigned long	st_ctime;
-   unsigned long	st_ctime_nsec;
-   long				__unused[3];
+   long long            st_blocks;
+   long                 st_atime;
+   unsigned long        st_atime_nsec;
+   long                 st_mtime;
+   unsigned long int    st_mtime_nsec;
+   long                 st_ctime;
+   unsigned long int    st_ctime_nsec;
+   unsigned long int    __unused4;
+   unsigned long int    __unused5;
 };
 
+
 //----------------------------------------------------------------------
 // From linux-2.6.9/include/asm-ppc/statfs.h
 //----------------------------------------------------------------------
@@ -572,14 +613,14 @@ extern unsigned int __VKI_invalid_size_argument_for_IOC;
 //#define VKI_TIOCGLCKTRMIOS	0x5456
 //#define VKI_TIOCSLCKTRMIOS	0x5457
 //#define VKI_TIOCSERGSTRUCT	0x5458 /* For debugging only */
-//#define VKI_TIOCSERGETLSR	0x5459 /* Get line status register */
-//  /* ioctl (fd, VKI_TIOCSERGETLSR, &result) where result may be as below */
+#define VKI_TIOCSERGETLSR	0x5459 /* Get line status register */
+  /* ioctl (fd, VKI_TIOCSERGETLSR, &result) where result may be as below */
 //# define VKI_TIOCSER_TEMT	0x01   /* Transmitter physically empty */
 //#define VKI_TIOCSERGETMULTI	0x545A /* Get multiport config  */
 //#define VKI_TIOCSERSETMULTI	0x545B /* Set multiport config */
 
 //#define VKI_TIOCMIWAIT	0x545C  /* wait for a change on serial input line(s) */
-//#define VKI_TIOCGICOUNT	0x545D  /* read serial port inline interrupt counts */
+#define VKI_TIOCGICOUNT		0x545D  /* read serial port inline interrupt counts */
 
 //----------------------------------------------------------------------
 // From linux-2.6.9/include/asm-ppc/poll.h
@@ -867,17 +908,6 @@ struct vki_shminfo64 {
 //.. ///* We need to pay attention to this, because it mmaps memory */
 //.. //#define VKI_DRM_IOCTL_MAP_BUFS		_VKI_IOWR('d', 0x19, struct vki_drm_buf_map)
 
-//.. //----------------------------------------------------------------------
-//.. // From linux-2.6.9/include/asm-i386/ptrace.h
-//.. //----------------------------------------------------------------------
-//.. 
-//.. #define VKI_PTRACE_GETREGS			12
-//.. #define VKI_PTRACE_SETREGS			13
-//.. #define VKI_PTRACE_GETFPREGS		14
-//.. #define VKI_PTRACE_SETFPREGS		15
-//.. #define VKI_PTRACE_GETFPXREGS		18
-//.. #define VKI_PTRACE_SETFPXREGS		19
-
 //----------------------------------------------------------------------
 // And that's it!
 //----------------------------------------------------------------------
